@@ -13,25 +13,15 @@ $d->push_back($d->factory(data => 'second'));
 $d->push_back($d->factory(data => 'third'));
 $d->push_back($d->factory(data => 'fourth'));
 $d->push_back($d->factory(data => 'fifth'));
-::foreach($d->begin(), $d->end(), MyPrint->new());
+for_each($d->begin(), $d->end(), ptr_fun('::myprint'));
 print '$d->push_front($d->factory(data => \'seventh\'));', "\n";
 $d->push_front($d->factory(data => 'seventh'));
-::foreach($d->begin(), $d->end(), MyPrint->new());
+for_each($d->begin(), $d->end(), ptr_fun('::myprint'));
 $d->pop_front();
 print '$d->pop_front();', "\n";
-::foreach($d->begin(), $d->end(), MyPrint->new());
+for_each($d->begin(), $d->end(), ptr_fun('::myprint'));
 $d->pop_back();
 print '$d->pop_back();', "\n";
-::foreach($d->begin(), $d->end(), MyPrint->new());
-# ----------------------------------------------------------------------------------------------------
-{
-	package MyPrint;
-	use base qw(Class::STL::Utilities::FunctionObject::UnaryFunction);
-	sub function_operator
-	{
-		my $self = shift;
-		my $arg = shift;
-		print "Data:", $arg->data(), "\n";
-	}
-}
-# ----------------------------------------------------------------------------------------------------
+for_each($d->begin(), $d->end(), ptr_fun('::myprint'));
+
+sub myprint { print "Data:", @_, "\n"; }
